@@ -41,14 +41,17 @@ pub trait CSharpTypeConverter {
     }
 
     fn has_overloadable(&self, signature: &FunctionSignature) -> bool {
-        signature.params().iter().any(|x| matches!(x.the_type(), CType::Pattern(TypePattern::Slice(_))))
+        signature
+            .params()
+            .iter()
+            .any(|x| matches!(x.the_type(), CType::Pattern(TypePattern::Slice(_) | TypePattern::SliceMut(_))))
     }
 
     fn pattern_to_native_in_signature(&self, param: &Parameter, _signature: &FunctionSignature) -> String {
         match param.the_type() {
             CType::Pattern(p) => match p {
                 TypePattern::AsciiPointer => "string".to_string(),
-                TypePattern::NamedCallback(x) => self.fnpointer_to_typename(x.fnpointer()),
+                TypePattern::NamedCallback(x) => x.name().to_string(),
                 TypePattern::SuccessEnum(e) => self.enum_to_typename(e.the_enum()),
                 TypePattern::Slice(p) => {
                     let element_type = p
@@ -73,7 +76,7 @@ pub trait CSharpTypeConverter {
                 }
 
                 TypePattern::Option(e) => self.composite_to_typename(e),
-                TypePattern::Bool => self.to_typespecifier_in_param(&p.fallback_type()),
+                TypePattern::Bool => "FFIBool".to_string(),
                 TypePattern::APIVersion => self.to_typespecifier_in_param(&p.fallback_type()),
             },
             x => self.to_typespecifier_in_param(x),
@@ -112,7 +115,7 @@ pub trait CSharpTypeConverter {
                 TypePattern::SliceMut(e) => self.composite_to_typename(e),
                 TypePattern::Option(e) => self.composite_to_typename(e),
                 TypePattern::NamedCallback(e) => self.named_callback_to_typename(e),
-                TypePattern::Bool => self.to_typespecifier_in_field(&x.fallback_type(), field, composite),
+                TypePattern::Bool => "FFIBool".to_string(),
                 TypePattern::APIVersion => self.to_typespecifier_in_field(&x.fallback_type(), field, composite),
             },
         }
@@ -148,7 +151,7 @@ pub trait CSharpTypeConverter {
                 TypePattern::SliceMut(x) => self.composite_to_typename(x),
                 TypePattern::Option(x) => self.composite_to_typename(x),
                 TypePattern::NamedCallback(x) => self.named_callback_to_typename(x),
-                TypePattern::Bool => self.to_typespecifier_in_param(&x.fallback_type()),
+                TypePattern::Bool => "FFIBool".to_string(),
                 TypePattern::APIVersion => self.to_typespecifier_in_param(&x.fallback_type()),
             },
         }
@@ -171,7 +174,7 @@ pub trait CSharpTypeConverter {
                 TypePattern::SliceMut(x) => self.composite_to_typename(x),
                 TypePattern::Option(x) => self.composite_to_typename(x),
                 TypePattern::NamedCallback(x) => self.named_callback_to_typename(x),
-                TypePattern::Bool => self.to_typespecifier_in_rval(&x.fallback_type()),
+                TypePattern::Bool => "FFIBool".to_string(),
                 TypePattern::APIVersion => self.to_typespecifier_in_rval(&x.fallback_type()),
             },
         }
